/*
 * Read/write starbase data
 * part of the Host-Toolkit library
 */

#include <stdio.h>
#include "tkhost.h"
#include "tkh.h"
#include "tkiocomm.h"

BASEDATA *read_basedata(int bnum)
{
  FILE *handle;
  BASEDATA *str;
  int k;

  handle=mkopen("bdata.hst",false,'r');
  if (!handle) return NULL;

  if (bnum>NUM_PLANETS || bnum==0) {
    fclose(handle);
    return NULL;
  }

  fseek(handle,2+156*(bnum-1),SEEK_SET);

  str=new BASEDATA;
  if (!str) {
    fclose(handle);
    return NULL;
  }

  str->wID=readUB2(handle);
  str->wOwner=readUB2(handle);
  str->wDefence=readUB2(handle);
  str->wDamage=readUB2(handle);
  str->wTechEngine=readUB2(handle);
  str->wTechHull=readUB2(handle);
  str->wTechBeam=readUB2(handle);
  str->wTechTorp=readUB2(handle);
  for(k=1;k<10;k++) {
    str->wEngines[k]=readUB2(handle);
  }
  for(k=1;k<21;k++) {
    str->wHulls[k]=readUB2(handle);
  }
  for(k=1;k<11;k++) {
    str->wBeams[k]=readUB2(handle);
  }
  for(k=1;k<11;k++) {
    str->wTLaunchers[k]=readUB2(handle);
  }
  for(k=1;k<11;k++) {
    str->wTorps[k]=readUB2(handle);
  }
  str->wFighters=readUB2(handle);
  str->wFixID=readUB2(handle);
  str->wFixShip=readUB2(handle);
  str->wMission=readUB2(handle);
  str->wShip=readUB2(handle);
  str->wEngineLevel=readUB2(handle);
  str->wBeamLevel=readUB2(handle);
  str->wBeamOnShip=readUB2(handle);
  str->wTorpLevel=readUB2(handle);
  str->wTorpOnShip=readUB2(handle);
  str->wFighterOnShip=readUB2(handle);

  fclose(handle);
  return str;
}

bool write_basedata(int bnum,BASEDATA *str)
{
  FILE *handle;
  int k;
  bool v;

  handle=mkopen("bdata.hst",false,'w');
  if (!handle) return false;

  if (bnum>NUM_PLANETS || bnum==0) {
    fclose(handle);
    return false;
  }

  fseek(handle,2+156*(bnum-1),SEEK_SET);

  v=writeUB2(handle,str->wID);
  if (v) v=writeUB2(handle,str->wOwner);
  if (v) v=writeUB2(handle,str->wDefence);
  if (v) v=writeUB2(handle,str->wDamage);
  if (v) v=writeUB2(handle,str->wTechEngine);
  if (v) v=writeUB2(handle,str->wTechHull);
  if (v) v=writeUB2(handle,str->wTechBeam);
  if (v) v=writeUB2(handle,str->wTechTorp);
  for(k=1;k<10;k++) {
    if (v) v=writeUB2(handle,str->wEngines[k]);
  }
  for(k=1;k<21;k++) {
    if (v) v=writeUB2(handle,str->wHulls[k]);
  }
  for(k=1;k<11;k++) {
    if (v) v=writeUB2(handle,str->wBeams[k]);
  }
  for(k=1;k<11;k++) {
    if (v) v=writeUB2(handle,str->wTLaunchers[k]);
  }
  for(k=1;k<11;k++) {
    if (v) v=writeUB2(handle,str->wTorps[k]);
  }
  if (v) v=writeUB2(handle,str->wFighters);
  if (v) v=writeUB2(handle,str->wFixID);
  if (v) v=writeUB2(handle,str->wFixShip);
  if (v) v=writeUB2(handle,str->wMission);
  if (v) v=writeUB2(handle,str->wShip);
  if (v) v=writeUB2(handle,str->wEngineLevel);
  if (v) v=writeUB2(handle,str->wBeamLevel);
  if (v) v=writeUB2(handle,str->wBeamOnShip);
  if (v) v=writeUB2(handle,str->wTorpLevel);
  if (v) v=writeUB2(handle,str->wTorpOnShip);
  if (v) v=writeUB2(handle,str->wFighterOnShip);

  fclose(handle);
  return v;
}

BASEDATA * read_all_basedata()
{
  FILE *handle;
  BASEDATA *str;
  int k,bnum;

  handle=mkopen("bdata.hst",false,'r');
  if (!handle) return NULL;

  str=new BASEDATA [501];
  if (!str) {
    fclose(handle);
    return NULL;
  }

  fseek(handle,2,SEEK_SET);

  for (bnum=1;bnum<=500;bnum++) {
    str[bnum].wID=readUB2(handle);
    str[bnum].wOwner=readUB2(handle);
    str[bnum].wDefence=readUB2(handle);
    str[bnum].wDamage=readUB2(handle);
    str[bnum].wTechEngine=readUB2(handle);
    str[bnum].wTechHull=readUB2(handle);
    str[bnum].wTechBeam=readUB2(handle);
    str[bnum].wTechTorp=readUB2(handle);
    for(k=1;k<10;k++) {
      str[bnum].wEngines[k]=readUB2(handle);
    }
    for(k=1;k<21;k++) {
      str[bnum].wHulls[k]=readUB2(handle);
    }
    for(k=1;k<11;k++) {
      str[bnum].wBeams[k]=readUB2(handle);
    }
    for(k=1;k<11;k++) {
      str[bnum].wTLaunchers[k]=readUB2(handle);
    }
    for(k=1;k<11;k++) {
      str[bnum].wTorps[k]=readUB2(handle);
    }
    str[bnum].wFighters=readUB2(handle);
    str[bnum].wFixID=readUB2(handle);
    str[bnum].wFixShip=readUB2(handle);
    str[bnum].wMission=readUB2(handle);
    str[bnum].wShip=readUB2(handle);
    str[bnum].wEngineLevel=readUB2(handle);
    str[bnum].wBeamLevel=readUB2(handle);
    str[bnum].wBeamOnShip=readUB2(handle);
    str[bnum].wTorpLevel=readUB2(handle);
    str[bnum].wTorpOnShip=readUB2(handle);
    str[bnum].wFighterOnShip=readUB2(handle);
  }

  fclose(handle);
  return str;
}
