/*
 * Read hull-list and *spec data
 * part of the Host-Toolkit library
 */

#include <stdio.h>
#include "tkhost.h"
#include "tkh.h"
#include "tkiocomm.h"

TRUEHULL *read_truehull(int hnum,int playernum)
{
  FILE *handle;
  TRUEHULL *str;

  handle=mkopen("truehull.dat",true,'r');
  if (!handle) return NULL;

  if (hnum>20 || hnum<1) {
    fclose(handle);
    return NULL;
  }

  fseek(handle,2*(hnum-1)+40*(playernum-1),SEEK_SET);

  str=new TRUEHULL;
  if (!str) {
    fclose(handle);
    return NULL;
  }

  str->wHullnum=readUB2(handle);

  fclose(handle);
  return str;
}

HULLSPEC *read_hullspec(int hnum)
{
  FILE *handle;
  HULLSPEC *str;

  handle=mkopen("hullspec.dat",true,'r');
  if (!handle) return NULL;

  if (hnum>MAXHULL || hnum<1) {
    fclose(handle);
    return NULL;
  }

  fseek(handle,60*(hnum-1),SEEK_SET);

  str=new HULLSPEC;
  if (!str) {
    fclose(handle);
    return NULL;
  }

  str->sName=readXString(handle,30);
  str->wPicture=readUB2(handle);
  str->wDamagePic=readUB2(handle);
  str->cargo.tri=readUB2(handle);
  str->cargo.dur=readUB2(handle);
  str->cargo.mol=readUB2(handle);
  str->wFuel=readUB2(handle);
  str->wCrew=readUB2(handle);
  str->wEngines=readUB2(handle);
  str->wMass=readUB2(handle);
  str->wTechLevel=readUB2(handle);
  str->wCargo=readUB2(handle);
  str->wFighterBays=readUB2(handle);
  str->wTorpsBays=readUB2(handle);
  str->wBeams=readUB2(handle);
  str->cargo.money=readUB2(handle);

  fclose(handle);
  return str;
}

BEAMSPEC *read_beamspec(int bnum)
{
  FILE *handle;
  BEAMSPEC *str;

  handle=mkopen("beamspec.dat",true,'r');
  if (!handle) return NULL;

  if (bnum>10) {
    fclose(handle);
    return NULL;
  }

  fseek(handle,36*(bnum-1),SEEK_SET);

  str=new BEAMSPEC;
  if (!str) {
    fclose(handle);
    return NULL;
  }

  str->sName=readXString(handle,20);
  str->cargo.money=readUB2(handle);
  str->cargo.tri=readUB2(handle);
  str->cargo.dur=readUB2(handle);
  str->cargo.mol=readUB2(handle);
  str->wMass=readUB2(handle);
  str->wTechLevel=readUB2(handle);
  str->wKill=readUB2(handle);
  str->wExplosive=readUB2(handle);

  fclose(handle);
  return str;
}

TORPSPEC *read_torpspec(int tnum)
{
  FILE *handle;
  TORPSPEC *str;

  handle=mkopen("torpspec.dat",true,'r');
  if (!handle) return NULL;

  if (tnum>10) {
    fclose(handle);
    return NULL;
  }

  fseek(handle,38*(tnum-1),SEEK_SET);

  str=new TORPSPEC;
  if (!str) {
    fclose(handle);
    return NULL;
  }

  str->sName=readXString(handle,20);
  str->wCostTorp=readUB2(handle);
  str->cargo.money=readUB2(handle);
  str->cargo.tri=readUB2(handle);
  str->cargo.dur=readUB2(handle);
  str->cargo.mol=readUB2(handle);
  str->wMass=readUB2(handle);
  str->wTechLevel=readUB2(handle);
  str->wKill=readUB2(handle);
  str->wExplosive=readUB2(handle);

  fclose(handle);
  return str;
}

ENGSPEC *read_engspec(int e_num)
{
  FILE *handle;
  ENGSPEC *str;
  int k;

  handle=mkopen("engspec.dat",true,'r');
  if (!handle) return NULL;

  if (e_num>9) {
    fclose(handle);
    return NULL;
  }

  fseek(handle,66*(e_num-1),SEEK_SET);

  str=new ENGSPEC;
  if (!str) {
    fclose(handle);
    return NULL;
  }

  str->sName=readXString(handle,20);
  str->cargo.money=readUB2(handle);
  str->cargo.tri=readUB2(handle);
  str->cargo.dur=readUB2(handle);
  str->cargo.mol=readUB2(handle);
  str->wTechLevel=readUB2(handle);
  for (k=1;k<10;k++) {
    str->lFuelUse[k]=readUB4(handle);
  }

  fclose(handle);
  return str;
}

TRUEHULL *read_all_truehull(int pn)
{
  FILE *handle;
  TRUEHULL *str;
  int rnum;

  str=new TRUEHULL [21];
  if (!str) return NULL;

  handle=mkopen("truehull.dat",true,'r');
  if (!handle) return NULL;

  fseek(handle,40*(pn-1),SEEK_SET);

  for(rnum=1;rnum<=20;rnum++) {
    str[rnum].wHullnum=readUB2(handle);
  }

  fclose(handle);
  return str;
}

HULLSPEC *read_all_hullspec()
{
  FILE *handle;
  HULLSPEC *str;
  int hnum;

  str=new HULLSPEC [MAXHULL+1];
  if (!str) return NULL;

  handle=mkopen("hullspec.dat",true,'r');
  if (!handle) return NULL;

  for(hnum=1;hnum<=MAXHULL;hnum++) {
    str[hnum].sName=readXString(handle,30);
    str[hnum].wPicture=readUB2(handle);
    str[hnum].wDamagePic=readUB2(handle);
    str[hnum].cargo.tri=readUB2(handle);
    str[hnum].cargo.dur=readUB2(handle);
    str[hnum].cargo.mol=readUB2(handle);
    str[hnum].wFuel=readUB2(handle);
    str[hnum].wCrew=readUB2(handle);
    str[hnum].wEngines=readUB2(handle);
    str[hnum].wMass=readUB2(handle);
    str[hnum].wTechLevel=readUB2(handle);
    str[hnum].wCargo=readUB2(handle);
    str[hnum].wFighterBays=readUB2(handle);
    str[hnum].wTorpsBays=readUB2(handle);
    str[hnum].wBeams=readUB2(handle);
    str[hnum].cargo.money=readUB2(handle);
  }

  fclose(handle);
  return str;
}

BEAMSPEC *read_all_beamspec()
{
  FILE *handle;
  BEAMSPEC *str;
  int bnum;

  str=new BEAMSPEC [11];
  if (!str) return NULL;

  handle=mkopen("beamspec.dat",true,'r');
  if (!handle) return NULL;

  for(bnum=1;bnum<11;bnum++) {
    str[bnum].sName=readXString(handle,20);
    str[bnum].cargo.money=readUB2(handle);
    str[bnum].cargo.tri=readUB2(handle);
    str[bnum].cargo.dur=readUB2(handle);
    str[bnum].cargo.mol=readUB2(handle);
    str[bnum].wMass=readUB2(handle);
    str[bnum].wTechLevel=readUB2(handle);
    str[bnum].wKill=readUB2(handle);
    str[bnum].wExplosive=readUB2(handle);
  }

  fclose(handle);
  return str;
}

TORPSPEC *read_all_torpspec()
{
  FILE *handle;
  TORPSPEC *str;
  int tnum;

  str=new TORPSPEC [11];
  if (!str) return NULL;

  handle=mkopen("torpspec.dat",true,'r');
  if (!handle) return NULL;

  for(tnum=1;tnum<11;tnum++) {
    str[tnum].sName=readXString(handle,20);
    str[tnum].wCostTorp=readUB2(handle);
    str[tnum].cargo.money=readUB2(handle);
    str[tnum].cargo.tri=readUB2(handle);
    str[tnum].cargo.dur=readUB2(handle);
    str[tnum].cargo.mol=readUB2(handle);
    str[tnum].wMass=readUB2(handle);
    str[tnum].wTechLevel=readUB2(handle);
    str[tnum].wKill=readUB2(handle);
    str[tnum].wExplosive=readUB2(handle);
  }

  fclose(handle);
  return str;
}

ENGSPEC *read_all_engspec()
{
  FILE *handle;
  ENGSPEC *str;
  int k;
  int e_num;

  str=new ENGSPEC [10];
  if (!str) return NULL;

  handle=mkopen("engspec.dat",true,'r');
  if (!handle) return NULL;

  for(e_num=1;e_num<10;e_num++) {
    str[e_num].sName=readXString(handle,20);
    str[e_num].cargo.money=readUB2(handle);
    str[e_num].cargo.tri=readUB2(handle);
    str[e_num].cargo.dur=readUB2(handle);
    str[e_num].cargo.mol=readUB2(handle);
    str[e_num].wTechLevel=readUB2(handle);
    for (k=1;k<10;k++) {
      str[e_num].lFuelUse[k]=readUB4(handle);
    }
  }

  fclose(handle);
  return str;
}
