/*
 * Read/write UFO data
 * part of the Host-Toolkit library
 */

#include <stdio.h>
#include "tkhost.h"
#include "tkh.h"
#include "tkiocomm.h"

UFO_DATA *read_ufo_data(int inum)
{
  FILE *handle;
  UFO_DATA *str;

  handle=mkopen("ufo.hst",false,'r');
  if (!handle) return NULL;

  if (inum>100 || inum<1) {
    fclose(handle);
    return NULL;
  }

  fseek(handle,78*(inum-1),SEEK_SET);

  str=new UFO_DATA;
  if (!str) {
    fclose(handle);
    return NULL;
  }

  str->wMarker=readUB2(handle);
  str->sNm=readXString(handle,20);
  str->Info1=readXString(handle,20);
  str->Info2=readXString(handle,20);
  str->pos.x=readSB2(handle);
  str->pos.y=readSB2(handle);
  str->wWarp=readUB2(handle);
  str->wHeading=readUB2(handle);
  str->wRangeP=readUB2(handle);
  str->wRangeS=readUB2(handle);
  str->wEdge=readUB2(handle);
  str->wParent=readUB2(handle);

  fclose(handle);
  return str;
}

UFO_DATA *read_all_ufo_data()
{
  FILE *handle;
  UFO_DATA *str;
  int inum;

  handle=mkopen("ufo.hst",false,'r');
  if (!handle) return NULL;

  str=new UFO_DATA [101];
  if (!str) {
    fclose(handle);
    return NULL;
  }

  for(inum=1;inum<=100;inum++) {
    str[inum].wMarker=readUB2(handle);
    str[inum].sNm=readXString(handle,20);
    str[inum].Info1=readXString(handle,20);
    str[inum].Info2=readXString(handle,20);
    str[inum].pos.x=readSB2(handle);
    str[inum].pos.y=readSB2(handle);
    str[inum].wWarp=readUB2(handle);
    str[inum].wHeading=readUB2(handle);
    str[inum].wRangeP=readUB2(handle);
    str[inum].wRangeS=readUB2(handle);
    str[inum].wEdge=readUB2(handle);
    str[inum].wParent=readUB2(handle);
  }

  fclose(handle);
  return str;
}

bool write_ufo_data(int inum,UFO_DATA *str)
{
  FILE *handle;
  bool v=true;

  handle=mkopen("ufo.hst",true,'w');
  if (!handle) return NULL;

  if (inum>100 || inum<1) {
    fclose(handle);
    return NULL;
  }

  fseek(handle,78*(inum-1),SEEK_SET);

  v=writeUB2(handle,str->wMarker);
  if (v) v=writeXString(handle,str->sNm,20);
  if (v) v=writeXString(handle,str->Info1,20);
  if (v) v=writeXString(handle,str->Info2,20);
  if (v) v=writeSB2(handle,str->pos.x);
  if (v) v=writeSB2(handle,str->pos.y);
  if (v) v=writeUB2(handle,str->wWarp);
  if (v) v=writeUB2(handle,str->wHeading);
  if (v) v=writeUB2(handle,str->wRangeP);
  if (v) v=writeUB2(handle,str->wRangeS);
  if (v) v=writeUB2(handle,str->wEdge);
  if (v) v=writeUB2(handle,str->wParent);

  fclose(handle);
  return v;
}
