#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>


#if INT_MIN < -2147483647 && INT_MAX >= 2147483647

typedef int            SB4;
typedef unsigned int   UB4;

#else

typedef long           SB4;
typedef unsigned long  UB4

#endif

#if SHRT_MIN < -32767 && SHRT_MAX >= 32767

typedef short          SB2;
typedef unsigned short UB2;

#else

typedef int            SB2;
typedef unsigned int   UB2

#endif

typedef signed char    SB1;
typedef unsigned char  UB1;

#include "xstring.h"
#include "tkiocomm.h"

extern XString game_dir;

SB4
readSB4(FILE *handle)
{
  SB4 var=0;
  UB4 buffer;
  int k;

  for (k=0;k<4;k++) {
    buffer=(UB4)fgetc(handle);
    var += buffer << (8*k);
  }
  return var;
}

UB4
readUB4(FILE *handle)
{
  UB4 var=0;
  UB4 buffer;
  int k;

  for (k=0;k<4;k++) {
    buffer=fgetc(handle);
    var += buffer << (8*k);
  }
  return var;
}

SB2
readSB2(FILE *handle)
{
  SB2 var=0;
  UB2 buffer;
  int k;

  for (k=0;k<2;k++) {
    buffer=fgetc(handle);
    var += buffer << (8*k);
  }
  return var;
}

UB2
readUB2(FILE *handle)
{
  UB2 var=0;
  UB2 buffer;
  int k;

  for (k=0;k<2;k++) {
    buffer=fgetc(handle);
    var += buffer << (8*k);
  }
  return var;
}

SB1
readSB1(FILE *handle)
{
  SB1 var;

  var=fgetc(handle);
  return var;
}

UB1
readUB1(FILE *handle)
{
  UB1 var;

  var=fgetc(handle);
  return var;
}

XString
readXString(FILE *handle,int maxlen)
{
  XString var;
  char *buffer;

  buffer=(char *)malloc(maxlen+1);
  if (!buffer) {
    //panic
  }
  buffer[0]=0;
  fread(buffer,maxlen,1,handle);
  buffer[maxlen]=0;
  var=buffer;
  var.TrimRight();
  free(buffer);
  return var;
}

XString
readXStringD(FILE *handle,int maxlen)
{
  XString var;
  char *buffer;
  int k;

  buffer=(char *)malloc(maxlen+1);
  if (!buffer) {
    //panic
  }
  buffer[0]=0;
  fread(buffer,maxlen,1,handle);
  buffer[maxlen]=0;
  for (k=0;k<maxlen;k++) {
    buffer[k]-=13;
    if (buffer[k]==0) buffer[k]=' ';
  }
  var=buffer;
  var.TrimRight();
  free(buffer);
  return var;
}

bool
writeSB4(FILE *handle,SB4 parm)
{
  UB1 buffer;
  int k;
  for (k=0;k<4;k++) {
    buffer=parm & 0xFF;
    parm >>= 8;
    if (fwrite(&buffer,1,1,handle) < 1) {
      return false;
    }
  }
  return true;
}

bool
writeUB4(FILE *handle,UB4 parm)
{
  UB1 buffer;
  int k;
  for (k=0;k<4;k++) {
    buffer=parm & 0xFF;
    parm >>= 8;
    if (fwrite(&buffer,1,1,handle) < 1) {
      return false;
    }
  }
  return true;
}

bool
writeSB2(FILE *handle,SB2 parm)
{
  UB1 buffer;
  buffer=parm & 0xFF;
  if (fwrite(&buffer,1,1,handle) < 1) {
    return false;
  }
  buffer=parm >> 8;
  if (fwrite(&buffer,1,1,handle) < 1) {
    return false;
  }
  return true;
}

bool
writeUB2(FILE *handle,UB2 parm)
{
  UB1 buffer;
  buffer=parm & 0xFF;
  if (fwrite(&buffer,1,1,handle) < 1) {
    return false;
  }
  buffer=parm >> 8;
  if (fwrite(&buffer,1,1,handle) < 1) {
    return false;
  }
  return true;
}

bool
writeSB1(FILE *handle,SB1 parm)
{
  if (fwrite(&parm,1,1,handle) < 1) {
    return false;
  }
  return true;
}

bool
writeUB1(FILE *handle,UB1 parm)
{
  if (fwrite(&parm,1,1,handle) < 1) {
    return false;
  }
  return true;
}

bool
writeXString(FILE *handle,int maxlen,XString parm)
{
  XString pad;
  char *buffer;

  if (parm.Len()>maxlen) {
    parm=parm.Left(maxlen);
  }
  if (parm.Len()<maxlen) {
    pad=XString(' ',maxlen-parm.Len());
    parm+=pad;
  }

  buffer=(char *)malloc(maxlen+1);
  if (!buffer) {
    //panic
  }
  strcpy(buffer,(const char *)parm);
  if (fwrite(buffer,maxlen,1,handle) < 1) {
    return false;
  }
  return true;
}

bool
writeXStringD(FILE *handle,int maxlen,XString parm)
{
  XString pad;
  char *buffer;
  int k;

  if (parm.Len()>maxlen) {
    parm=parm.Left(maxlen);
  }
  if (parm.Len()<maxlen) {
    pad=XString(' ',maxlen-parm.Len());
    parm+=pad;
  }

  buffer=(char *)malloc(maxlen+1);
  if (!buffer) {
    //panic
  }
  strcpy(buffer,(const char *)parm);
  for (k=0;k<maxlen;k++) {
    buffer[k]+=13;
  }
  if (fwrite(buffer,maxlen,1,handle) < 1) {
    return false;
  }
  return true;
}

FILE *mkxopen(XString fn,bool trylocal,char mode)
{
  XString rn;
  FILE *h;
  fpos_t l;
  char rmode[4];

  if (mode=='w' && trylocal) { // copy generic if no file available
    rn=game_dir | "\\" | fn;
    h=fopen((const char *)rn,"rb");
    if (h) {
      fseek(h,0,SEEK_END);
      l=ftell(h);
      fclose(h);
      if (l==0) h=NULL;
    }
    if (!h) {
      XString sc;
      sc="copy " | fn | " " | rn | " >nul";
      system((const char *)sc);
    }
  }
  strcpy(rmode,"rb");
  if (mode=='w') strcpy(rmode,"rb+");
  if (mode=='o') strcpy(rmode,"wb");
  rn=game_dir | "\\" | fn;
  h=fopen((const char *)rn,rmode);
  if (!h) {
    if (mode=='r' && trylocal) {
      rn=fn;
      h=fopen((const char *)rn,rmode);
    }
  }

  return h;
}

