/*
 * Read/write base data
 * part of the Player-Toolkit package
 */

#include <stdio.h>
#include "tkplayer.h"
#include "tkp.h"
#include "tkiocomm.h"

static UB2 baseindex[500];
static bool baseabs;

int num_bases(bool datfile)
{
  FILE *handle;
  int ns;

  if (datfile) {
    handle=mkopen("bdata&p.dat",false,'r');
  } else {
    handle=mkopen("bdata&p.dis",false,'r');
  }
  if (!handle) return 0;

  ns=(int)readUB2(handle);
  fclose(handle);
  return ns;
}

BASEDATA *read_basedata(int snum,bool datfile)
{
  if (baseabs==false) {
    BASEDATA *tstr;
    int s,k;
    k=num_bases(datfile);
    for(s=1;s<=k;s++) {
      tstr=read_basedata_A(s,datfile);
      if (tstr) {
        baseindex[tstr->wID-1]=s;
        delete tstr;
      }
    }
    baseabs=true;
  }

  return read_basedata_A(baseindex[snum-1],datfile);
}

BASEDATA *read_basedata_A(int bnum,bool datfile)
{
  FILE *handle;
  BASEDATA *str;
  UB2 ns;
  int k;

  if (datfile) {
    handle=mkopen("bdata&p.dat",false,'r');
  } else {
    handle=mkopen("bdata&p.dis",false,'r');
  }
  if (!handle) return NULL;

  ns=readUB2(handle);
  if (bnum>ns || bnum==0) {
    fclose(handle);
    return NULL;
  }

  fseek(handle,2+156*(bnum-1),SEEK_SET);

  str=new BASEDATA;
  if (!str) {
    fclose(handle);
    return NULL;
  }

  str->wID=readUB2(handle);
  str->wOwner=readUB2(handle);
  str->wDefence=readUB2(handle);
  str->wDamage=readUB2(handle);
  str->wTechEngine=readUB2(handle);
  str->wTechHull=readUB2(handle);
  str->wTechBeam=readUB2(handle);
  str->wTechTorp=readUB2(handle);
  for(k=1;k<10;k++) {
    str->wEngines[k]=readUB2(handle);
  }
  for(k=1;k<21;k++) {
    str->wHulls[k]=readUB2(handle);
  }
  for(k=1;k<11;k++) {
    str->wBeams[k]=readUB2(handle);
  }
  for(k=1;k<11;k++) {
    str->wTLaunchers[k]=readUB2(handle);
  }
  for(k=1;k<11;k++) {
    str->wTorps[k]=readUB2(handle);
  }
  str->wFighters=readUB2(handle);
  str->wFixID=readUB2(handle);
  str->wFixShip=readUB2(handle);
  str->wMission=readUB2(handle);
  str->wShip=readUB2(handle);
  str->wEngineLevel=readUB2(handle);
  str->wBeamLevel=readUB2(handle);
  str->wBeamOnShip=readUB2(handle);
  str->wTorpLevel=readUB2(handle);
  str->wTorpOnShip=readUB2(handle);
  str->wFighterOnShip=readUB2(handle);

  fclose(handle);
  return str;
}

bool write_basedata(int snum,BASEDATA *str)
{
  if (baseabs==false) {
    BASEDATA *tstr;
    tstr=read_basedata(1);
    if (tstr) delete tstr;
  }

  return write_basedata_A(baseindex[snum-1],str);
}

bool write_basedata_A(int bnum,BASEDATA *str)
{
  FILE *handle;
  UB2 ns;
  int k;
  bool v;

  handle=mkopen("bdata&p.dat",false,'w');
  if (!handle) return false;

  ns=readUB2(handle);
  if (bnum>ns || bnum==0) {
    fclose(handle);
    return false;
  }

  fseek(handle,2+156*(bnum-1),SEEK_SET);

  v=writeUB2(handle,str->wID);
  if (v) v=writeUB2(handle,str->wOwner);
  if (v) v=writeUB2(handle,str->wDefence);
  if (v) v=writeUB2(handle,str->wDamage);
  if (v) v=writeUB2(handle,str->wTechEngine);
  if (v) v=writeUB2(handle,str->wTechHull);
  if (v) v=writeUB2(handle,str->wTechBeam);
  if (v) v=writeUB2(handle,str->wTechTorp);
  for(k=1;k<10;k++) {
    if (v) v=writeUB2(handle,str->wEngines[k]);
  }
  for(k=1;k<21;k++) {
    if (v) v=writeUB2(handle,str->wHulls[k]);
  }
  for(k=1;k<11;k++) {
    if (v) v=writeUB2(handle,str->wBeams[k]);
  }
  for(k=1;k<11;k++) {
    if (v) v=writeUB2(handle,str->wTLaunchers[k]);
  }
  for(k=1;k<11;k++) {
    if (v) v=writeUB2(handle,str->wTorps[k]);
  }
  if (v) v=writeUB2(handle,str->wFighters);
  if (v) v=writeUB2(handle,str->wFixID);
  if (v) v=writeUB2(handle,str->wFixShip);
  if (v) v=writeUB2(handle,str->wMission);
  if (v) v=writeUB2(handle,str->wShip);
  if (v) v=writeUB2(handle,str->wEngineLevel);
  if (v) v=writeUB2(handle,str->wBeamLevel);
  if (v) v=writeUB2(handle,str->wBeamOnShip);
  if (v) v=writeUB2(handle,str->wTorpLevel);
  if (v) v=writeUB2(handle,str->wTorpOnShip);
  if (v) v=writeUB2(handle,str->wFighterOnShip);

  fclose(handle);
  return v;
}

BASEDATA * read_all_basedata(bool datfile)
{
  FILE *handle;
  BASEDATA *str;
  int k,bnum,bmax;
  UB2 bid;

  bmax=num_bases(datfile);
  if (bmax==0) return NULL;

  if (datfile) {
    handle=mkopen("bdata&p.dat",false,'r');
  } else {
    handle=mkopen("bdata&p.dis",false,'r');
  }
  if (!handle) return NULL;

  str=new BASEDATA [501];
  if (!str) {
    fclose(handle);
    return NULL;
  }

  for(k=1;k<501;k++) {
    str[k].wID=0;
  }

  fseek(handle,2,SEEK_SET);

  for (bnum=1;bnum<=bmax;bnum++) {
    bid=readUB2(handle);
    str[bid].wID=bid;
    str[bid].wOwner=readUB2(handle);
    str[bid].wDefence=readUB2(handle);
    str[bid].wDamage=readUB2(handle);
    str[bid].wTechEngine=readUB2(handle);
    str[bid].wTechHull=readUB2(handle);
    str[bid].wTechBeam=readUB2(handle);
    str[bid].wTechTorp=readUB2(handle);
    for(k=1;k<10;k++) {
      str[bid].wEngines[k]=readUB2(handle);
    }
    for(k=1;k<21;k++) {
      str[bid].wHulls[k]=readUB2(handle);
    }
    for(k=1;k<11;k++) {
      str[bid].wBeams[k]=readUB2(handle);
    }
    for(k=1;k<11;k++) {
      str[bid].wTLaunchers[k]=readUB2(handle);
    }
    for(k=1;k<11;k++) {
      str[bid].wTorps[k]=readUB2(handle);
    }
    str[bid].wFighters=readUB2(handle);
    str[bid].wFixID=readUB2(handle);
    str[bid].wFixShip=readUB2(handle);
    str[bid].wMission=readUB2(handle);
    str[bid].wShip=readUB2(handle);
    str[bid].wEngineLevel=readUB2(handle);
    str[bid].wBeamLevel=readUB2(handle);
    str[bid].wBeamOnShip=readUB2(handle);
    str[bid].wTorpLevel=readUB2(handle);
    str[bid].wTorpOnShip=readUB2(handle);
    str[bid].wFighterOnShip=readUB2(handle);
  }

  fclose(handle);
  return str;
}

